#include <iostream>

#include <Windows.h>

#include <Varjo_motion_platform.h>
#include "PluginStatusCodes.hpp"
#include "ExampleMotionPlatform.hpp"

/**
 * Provide plugin types supported by the plugin library
 */
VARJO_API varjo_PluginType varjo_GetPluginType()
{  //
    std::cout << __FUNCTION__ << "(): " << std::endl;
    return varjo_PluginType_MotionPlatform;
}

/**
 * Provide the API version number supported by the plugin.
 * Use @ref VARJO_PLUGIN_API_VERSION macro to indicate supported API version number.
 */
VARJO_API uint64_t varjo_GetPluginAPIVersion()
{  //
    std::cout << __FUNCTION__ << "(): " << std::endl;
    return VARJO_PLUGIN_API_VERSION;
}

/**
 * @returns Null-terminated C-string containing plugin name
 */
VARJO_API const char* varjo_GetPluginName()
{  //
    std::cout << __FUNCTION__ << "(): " << std::endl;
    return "example motion platform plugin";
}

/**
 * Return a string description of a given status code
 */
VARJO_API const char* varjo_GetPluginStatusDescription(varjo_PluginStatus pluginStatus)
{  //
    std::cout << __FUNCTION__ << "(): " << std::endl;

    switch (pluginStatus) {
        case varjo_PluginStatus_OK: {
            return "No error";
        }
        case varjo_PluginStatus_NullPointer: {
            return "Function called with a null pointer";
        }
        case varjo_PluginStatus_TrackerInitializationError: {
            return "Couldn't initialize tracker";
        }
        case varjo_PluginStatus_TrackingNotInitialized: {
            return "Tracker is not intialized";
        }
        default: {
            break;
        }
    }

    return "Unknown error";
}

/**
 * Initialize plugin
 */
VARJO_API varjo_PluginStatus varjo_PluginInit(        //
    const varjo_PluginUtilityAPI* pluginUtilityAPI,   //
    varjo_PluginUtilitySession pluginUtilitySession,  //
    varjo_PluginContext* context)
{
    std::cout << __FUNCTION__ << "(): " << std::endl;
    if (!pluginUtilityAPI) {
        return varjo_PluginStatus_NullPointer;
    }

    ExampleMotionPlatform* motionPlatform = new ExampleMotionPlatform(  //
        pluginUtilityAPI, pluginUtilitySession);
    *context = reinterpret_cast<varjo_PluginContext>(motionPlatform);

    return varjo_PluginStatus_OK;
}

/**
 * Deinitialize plugin
 */
VARJO_API varjo_PluginStatus varjo_PluginShutdown(varjo_PluginContext* context)
{
    std::cout << __FUNCTION__ << "(): " << std::endl;
    ExampleMotionPlatform* motionPlatform = reinterpret_cast<ExampleMotionPlatform*>(*context);
    if (!motionPlatform) {
        return varjo_PluginStatus_NullPointer;
    }

    delete motionPlatform;
    *context = nullptr;

    return varjo_PluginStatus_OK;
}

/**
 * Update HMD pose
 */
VARJO_API varjo_PluginStatus varjo_UpdatePose(  //
    varjo_PluginContext context,                //
    varjo_DeviceType deviceType,                //
    const varjo_DevicePose* pose,                      //
    varjo_DevicePose* updatedPose)
{
    ExampleMotionPlatform* motionPlatform = reinterpret_cast<ExampleMotionPlatform*>(context);
    if (!motionPlatform || !pose || !updatedPose) {
        return varjo_PluginStatus_NullPointer;
    }

    return motionPlatform->updatePose(pose, updatedPose);
}

/**
 * Plugin DLL entry point
 */
BOOL WINAPI DllMain(  //
    IN HINSTANCE hDllHandle, IN DWORD nReason, IN LPVOID Reserved)
{
    switch (nReason) {
        case DLL_PROCESS_ATTACH:  //
            break;

        case DLL_PROCESS_DETACH:  //
            break;
    }

    return TRUE;
}
